/*
      Masterarbeit
      Programm für einen ESP32
      - ADNS3080 Test -
      Geschrieben von: Ricco Müller

      ADNS3080_Test
*/

//#include <WiFi.h>
#include <SPI.h>        // SPI
#include <mcp_can.h>    // Libary for MCP2515 CAN Driver
#include <ADNS3080.h>   // Libary for ADNS3080 Optical Flow Sensor

// Multi-Threading
TaskHandle_t Task1;
TaskHandle_t Task2;
TaskHandle_t Task3;

//// WiFi - Data
//const char* ssid = "NAME";
//const char* password = "PASSWORT";

// SPI class objects
SPIClass * hspi = new SPIClass(HSPI);   // For HSPI communication

// // Pin-Belegungen
// #define CS_PIN_CAN 15   // CS Pin for MCP2515   (HSPI)
// #define INT_PIN_CAN 21  // INT Pin for MCP2515
    // HSPI: CS=15 ; SO=12 ; SI=13 ; SCK=14

#define CS_PIN_ADNS 5   // CS Pin for ADNS3080  (VSPI - standard)
#define RESET_PIN 22    // Reset Pin for ADNS3080 (45 no pin -> not needed?)
    // VSPI: CS=5 ; SO=19 ; SI=23 ; SCK=18

// Konstanten deklaration
const float pi = 3.14159; 
const float deg2rad = pi/180;
const float rad2deg = 180/pi;

// // Variables deklaration
// long unsigned int rxId;     // received CAN-ID
// unsigned char len = 0;      // received DLC
// unsigned char rxBuf[8];     // received Data bytes (8) 
// char msgString[128];        // Array to store serial string
// byte data[8] = {0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07};  // Testdata

int8_t dx;    // Variable for displacement in x-direction since last reading
int x = 0;    // Variable for absolute displacement in x-direction
int8_t dy;    // Variable for displacement in y-direction since last reading
int y = 0;    // Variable for absolute displacement in y-direction
float beta = 0;    // Schwimmwinkel beta in degrees

// // MCP2515 class object (CAN-Driver)
// MCP_CAN CAN(hspi, CS_PIN_CAN);  

// ADNS3080 object
ADNS3080 <RESET_PIN, CS_PIN_ADNS> adns;

void setup()
{
  Serial.begin(115200);
  Serial.println("Willkommen im seriellen Monitor!");


  // // MCP2515
  // // Initialize MCP2515 running at 16MHz with a baudrate of 500kb/s and the masks and filters disabled.
  // if(CAN.begin(MCP_ANY, CAN_500KBPS, MCP_16MHZ) == CAN_OK)
  //   Serial.println("MCP2515 Initialized Successfully!");
  // else
  //   Serial.println("Error Initializing MCP2515...");

  // CAN.setMode(MCP_NORMAL);   // Set operation mode to normal so the MCP2515 sends acks to received data.
  
  // pinMode(INT_PIN_CAN, INPUT);   // Configuring pin for /INT input


  // ADNS3080 Setup
  adns.setup();


  //create a task that will be executed in the Task1_FLowSensor() function, with priority 1 and executed on core 0
  xTaskCreatePinnedToCore(
    Task1_FLowSensor,   /* Task function for Optical Flow Sensor reading. */
    "Task1",     /* name of task. */
    10000,       /* Stack size of task */
    NULL,        /* parameter of the task */
    1,           /* priority of the task */
    &Task1,      /* Task handle to keep track of created task */
    0);          /* pin task to core 0 */
  delay(500);

  //create a task that will be executed in the Task2_CAN() function, with priority 1 and executed on core 1
  xTaskCreatePinnedToCore(
    Task2_CAN,   /* Task function for CAN transmission. */
    "Task2",     /* name of task. */
    10000,       /* Stack size of task */
    NULL,        /* parameter of the task */
    1,           /* priority of the task */
    &Task2,      /* Task handle to keep track of created task */
    1);          /* pin task to core 1 */
  delay(500);

  //create a task that will be executed in the Task3_else() function, with priority 2 and executed on core 1
  xTaskCreatePinnedToCore(
    Task3_else,   /* Task function for everything else with lower priority. */
    "Task3",     /* name of task. */
    10000,       /* Stack size of task */
    NULL,        /* parameter of the task */
    2,           /* priority of the task */
    &Task3,      /* Task handle to keep track of created task */
    1);          /* pin task to core 1 */
  delay(500);
}

//Task1_FLowSensor:
void Task1_FLowSensor( void * pvParameters )
{
  Serial.print("Task1 running on core ");
  Serial.println(xPortGetCoreID());

  for (;;)  // forever loop
  {
    adns.displacement(&dx, &dy);
    x += dx;
    y += dy;
    beta = atan(float(dy) / float(dx)) * rad2deg;

    // Displacement:
    
    Serial.print( " dx=" );
    Serial.print( dx );
    Serial.print( " dy=" );
    Serial.print( dy );
    Serial.print( " x=" );
    Serial.print( x );
    Serial.print( " y=" );
    Serial.print( y );
    Serial.print( " beta=");
    Serial.print( beta );
    Serial.print( " degrees");
    Serial.println();

    vTaskDelay(500);
  }
}


//Task2_CAN:
void Task2_CAN( void * pvParameters )
{
  Serial.print("Task2 running on core ");
  Serial.println(xPortGetCoreID());

  for (;;)  // forever loop
  {
    // if(!digitalRead(INT_PIN_CAN))  // If CAN0_INT pin is low, read receive buffer
    // {
    //   Serial.println("Message ist daaaaaaa");
    //   CAN.readMsgBuf(&rxId, &len, rxBuf);   // Read data: len = DLC, buf = data byte(s)
      
    //   if((rxId & 0x80000000) == 0x80000000)   // Determine if ID is standard (11 bits) or extended (29 bits)
    //     sprintf(msgString, "Extended ID: 0x%.8lX  DLC: %1d  Data:", (rxId & 0x1FFFFFFF), len);
    //   else
    //     sprintf(msgString, "Standard ID: 0x%.3lX       DLC: %1d  Data:", rxId, len);
    
    //   Serial.print(msgString);
    
    //   if((rxId & 0x40000000) == 0x40000000){    // Determine if message is a remote request frame.
    //     sprintf(msgString, " REMOTE REQUEST FRAME");
    //     Serial.print(msgString);
    //   } else {
    //     for(byte i = 0; i<len; i++){
    //       sprintf(msgString, " 0x%.2X", rxBuf[i]);
    //       Serial.print(msgString);
    //     }
    //   } 
    //   Serial.println();

    //   vTaskDelay(500);
    //   // send data:  ID = 0x100, Standard CAN Frame, Data length = 8 bytes, 'data' = array of data bytes to send
    //   byte sndStat = CAN.sendMsgBuf(0x100, 0, 8, data);
    //   if(sndStat == CAN_OK){
    //     Serial.println("Message Sent Successfully!");
    //   } else {
    //     Serial.println("Error Sending Message...");
    //   }
    //   vTaskDelay(500);   // send data per 100ms
    // }
    // Serial.println("loop");
    vTaskDelay(1000);
  }
}


//Task3_else:
void Task3_else( void * pvParameters )
{
  Serial.print("Task3 running on core ");
  Serial.println(xPortGetCoreID());

  for (;;)  // forever loop
  {
    vTaskDelay(1000);
  }
}


void loop()
{
  return;
  //vTaskDelay(60 * 1000);
}
